process.on("SIGINT", () => process.exit()); //Fixes docker issue.
const http = require('http');
const https = require('https');
const fs = require('fs');
const configs = {
	'gtfs-nl.zip': {
		url: 'https://gtfs.ovapi.nl/gtfs-nl.zip',
		header: 'etag'
	}
}
const storagePath = process.env.STORAGE || "./data/"
console.log('using', storagePath, 'for storage. Configurable with STORAGE env variable.')
if(!fs.existsSync(storagePath)) fs.mkdirSync(storagePath);
const userAgent = process.env.USERAGENT || "cacheserver(cacheserver@rubend.nl)"
console.log('using', userAgent,'as userAgent. Configurable with the USERAGENT env variable.');
const userAgentOptions = {
	headers: {
		'User-Agent': userAgent
	}
}
function respondWithFile(res,filePath) {
	const readStream = fs.createReadStream(filePath);
	readStream.pipe(res);
}
http.createServer((req,res)=>{
	const key = req.url.substring(1);
	const config = configs[key];
	if(!config) {
		console.log("unknown config:", key)
		res.end("no config found.")
		return;
	}
	const headerPath = `${storagePath}${key}.header`;
	const filePath = `${storagePath}${key}.file`;
	const {url, header} = config;
	console.log('requesting', key, '=', url,'with header', header);
	const httpHandler = url.startsWith('https')?https:http
	httpHandler.request(url, { method: 'HEAD', ...userAgentOptions }, (headRes) => {
		const oldHeader = (fs.existsSync(headerPath)?fs.readFileSync(headerPath, 'utf8'):"NOT YET DOWNLOADED").trim();
		const newHeader = headRes.headers[header].trim();
		console.log('old header:',oldHeader, 'new header:', newHeader)
		if(oldHeader==newHeader) {
			console.log("file not changed, returning current file.")
			respondWithFile(res, filePath)
			return
		}
		httpHandler.get(url, userAgentOptions, downloadRes=>{
			const {statusCode}=downloadRes;
			console.log('code',statusCode);
			if(statusCode!=200) {
				res.writeHead(500);
				res.end("got error downloading file, check logs.");
				return;
			}
			const writeStream = fs.createWriteStream(filePath)
			downloadRes.pipe(writeStream);
			writeStream.on("finish", ()=>{
				fs.writeFileSync(headerPath, newHeader, 'utf8');
				console.log('download complete')
				respondWithFile(res, filePath)
			})
		})
	}).end();
}).listen(8080)